; #AutoIt3Wrapper_Au3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w- 7

; = Info ===========================================================================================
; Title           : AboutDialog.au3
; --------------------------------------------------------------------------------------------------
;
; Version (vnots) : 0.0.12.6
; Date            : 2021-04-12
; Author(s)       : Professor Bernd
;
; Requirements    :
; Language        :
; Description     : About-Dialog mit Kontaktmöglichkeiten und Danksagungen.
;                 : About dialog with contact options and credits.
; ==================================================================================================


; ==============================================================================
; ------------------------------------------------------------------------------
;
;         Anpassen des About Dialogs.  /  Customize the About dialog.
;
; Zum Anpassen des About Dialogs können die Bereiche benutzt werden, die mit
; Doppellinien gekennzeichnet sind, wie die über und unter diesem Hinweis.
;
; To customize the About dialog, you can use the areas marked with double
; lines, like the ones above and below this note.
; ------------------------------------------------------------------------------
; ==============================================================================


#NoTrayIcon
#include <File.au3>
#include <FontConstants.au3>
#include <GUIConstantsEx.au3>
#include <GuiRichEdit.au3>
#include <MsgBoxConstants.au3>
#include <StaticConstants.au3>
#include <WinAPIGdi.au3>
#include <WindowsConstants.au3>


; General settings.
; ==============================================================================
; ------------------------------------------------------------------------------

; Constants for captions and texts
Global Const $sGuiTitle = "About 'AboutDialog'"
Global Const $sPathRTF = _PathFull(".\About.rtf", @ScriptDir) ; Relative paths must NOT start with "\"!
Global Const $sVersionNum = "Version 1.0"

; Three of the dimensions of the additional controls. Only "Top" is set individually for each control.
Global Const $iCtrls_Left = 455
Global Const $iCtrls_Width = 180
Global Const $iCtrls_Height = 20

; BackColor for GUI and controls.
Global $iBackColor = 0xC0C0C0 ; medium light gray 0xC0C0C0

; Visited Color for LinkLabels.
Global Const $iLinkVisitedColor = 0xAC00A9

; RichEdit with or without borders.
Global $bRichEditFrameless = True
; ------------------------------------------------------------------------------
; ==============================================================================


; Wenn die RTF-Datei nicht existiert, eine Meldung zeigen und abbrechen.
; If the RTF file does not exist, show a message and abort.
If Not FileExists($sPathRTF) Then
  MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, $sGuiTitle, "File does not exist: " & @CRLF & @CRLF & $sPathRTF)
  Exit
EndIf

; Global variables.
Global $g_hParent ; Used e.g. in OnAutoItExit().
Global $g_hGui
Global $g_hRichEdit
Global $g_hUser32DLL = DllOpen("user32.dll")

Global $aLinkLabels[0][2]

; Caret verstecken, wenn das RichEdit mit dem Scrollbalken gescrollt wird.
; Hide caret when the RichEdit is scrolled with the scrollbar.
GUIRegisterMsg($WM_COMMAND, "WM_COMMAND")

; Klickbare Links im RichEdit ermöglichen. Wichtig: Die Links müssen beim Erstellen der RTF-Datei als Links eingerichtet werden!
; Enable clickable links in RichEdit. Important: The links must be set up as links when creating the RTF file!
GUIRegisterMsg($WM_NOTIFY, "WM_NOTIFY")

OnAutoItExitRegister('OnAutoItExit')

Main_Func()

; Clean up.
DllClose($g_hUser32DLL)


; ==============================================================================
; Main_Func
;
; Main function. ;-)
; ------------------------------------------------------------------------------
Func Main_Func()

  Local $iMsg, $idx
  Local $bHotkeyForTabIsSet = False

  $g_hParent = WinGetHandle("[active]")  ; Get parent window.

  ; Code nur ausführen, wenn das Handle zu einem PSPad-Fenster gehört.
  ; Execute code only if the handle belongs to a PSPad window.
;   If _WinAPI_IsClassName($g_hParent, "TfPSPad") Then ; <= Uncomment if needed and insert the desired window class.

    ; Parent-Fenster disablen. (Wird in "OnAutoItExit()" wieder enabled!)
    ; Achtung: Parent-Fenster bleibt unbenutzbar, wenn nicht wieder enabled wird!
    ;
    ; Disable parent window. (Will be enabled again in "OnAutoItExit()"!)
    ; => Attention: Parent window remains unusable, if not enabled again! <=
    WinSetState($g_hParent, "", @SW_DISABLE) ; Disable parent window.
    ; GUISetState(@SW_DISABLE, $g_hParent) ; Disable parent window for own AutoIt GUIs.

    ; GUI - Die GUI ist modal zum Parent-Fenster, hat kein TrayIcon und keinen
    ; Taskbar-Eintrag. (Wenn hParent verwendet wird, ist die GUI NICHT topmost!)
    ;
    ; GUI - The GUI is modal to the parent window, has no tray icon and no
    ; taskbar entry. (If hParent is used, the GUI is NOT topmost!)
    $g_hGui = GUICreate($sGuiTitle, 640, 360, -1, -1, BitOR($WS_CAPTION, $WS_POPUPWINDOW), -1, $g_hParent)
    GUISetBkColor($iBackColor, $g_hGui)

;     $g_hRichEdit = _GUICtrlRichEdit_Create($g_hGui, "", 10, 10, 440, 340, _
;         BitOR($ES_MULTILINE, $WS_HSCROLL, $WS_VSCROLL, $ES_AUTOVSCROLL, $ES_READONLY), $WS_EX_TRANSPARENT)

    If $bRichEditFrameless = False Then
      ; Create the RichEdit with borders.
      $g_hRichEdit = _GUICtrlRichEdit_Create($g_hGui, "", 10, 10, 440, 340, BitOR($ES_MULTILINE, $WS_HSCROLL, $WS_VSCROLL, $ES_READONLY))
    Else
      ; Create the RichEdit without borders.
      $g_hRichEdit = _GUICtrlRichEdit_Create($g_hGui, "", 10, 10, 440, 340, BitOR($ES_MULTILINE, $WS_HSCROLL, $WS_VSCROLL, $ES_READONLY), 0)
    EndIf

    ; Den Wert für die Hintergrundfarbe für das RichEdit von RGB zu BGR konvertieren.
    ; Convert the background color value for the RichEdit from RGB to BGR.
    _GUICtrlRichEdit_SetBkColor($g_hRichEdit, _WinAPI_SwitchColor($iBackColor))

    ; Set settings that allow RichEdit to recognize links and make them clickable.
    _GUICtrlRichEdit_AutoDetectURL($g_hRichEdit, True)
    _GUICtrlRichEdit_SetEventMask($g_hRichEdit, $ENM_LINK)

    ; Read RTF file and display it in RichEdit.
    _GUICtrlRichEdit_SetText($g_hRichEdit, FileRead($sPathRTF))


    ; Creating the additional controls (labels and buttons).
    ; ==========================================================================
    ; --------------------------------------------------------------------------

    ; Die zusätzlichen Conrols (labels und buttons) werden alle auf der rechten Seite
    ; des Dialog angezeigt und haben alle die gleichen Maße für "left", "width und
    ; "hight" - nur der Wert für "Top" wird für jedes Control individuell eingestellt.

    ; The additional conrols (labels and buttons) are all displayed on the right side
    ; of the dialog and all have the same dimensions for "left", "width" and "height"
    ; - only the value for "top" is set individually for each control.

    Local $lblVersionNum = GUICtrlCreateLabel($sVersionNum, $iCtrls_Left, 20, $iCtrls_Width, $iCtrls_Height, $SS_CENTER)
    GUICtrlSetFont($lblVersionNum, 10, $FW_BOLD, $GUI_FONTNORMAL, "Segoe UI")
;     GUICtrlSetBkColor($lblVersionNum, 0xFFFF00) ; Show background in color for debugging.

    Local $btnVersInfo = GUICtrlCreateButton("show versions info", $iCtrls_Left, 40, $iCtrls_Width)

    ; Label for the headline of LinkLabel-1.
    GUICtrlCreateLabel("Support", $iCtrls_Left, 80, $iCtrls_Width, $iCtrls_Height, $SS_CENTER)

    ; Syntax: CreateLinkLabel ( $_iTop, $_sCaption, $_sUrlOrMailTo )
    CreateLinkLabel(100, "support@info.com", "mailto:support@info.com?subject=Something")
    CreateLinkLabel(125, "even much better support", "mailto:example@info.com?subject=Something")
    CreateLinkLabel(220, "German AutoIt Forum", "https://autoit.de/board-list/")
    CreateLinkLabel(240, "English AutoIt Forum", "https://www.autoitscript.com/forum/")

    ; "Ok" button to close the about dialog.
    Local $btnOK = GUICtrlCreateButton("OK", $iCtrls_Left, 325, $iCtrls_Width)
    ; --------------------------------------------------------------------------
    ; ==========================================================================


    GUISetState(@SW_SHOW)

    DllCall($g_hUser32DLL, "int", "HideCaret", "hwnd", $g_hRichEdit)     ; <--- this hides caret initially

    While True

      $iMsg = GUIGetMsg()


      ; Insert click events for buttons and labels - but NOT for LinkLabels!
      ; ========================================================================
      ; ------------------------------------------------------------------------
      Switch $iMsg

        Case $GUI_EVENT_CLOSE, $btnOK
          _GUICtrlRichEdit_Destroy($g_hRichEdit) ; needed unless script crashes
          ; GUIDelete()   ; is OK too
          ExitLoop ; So that the script can execute commands, e.g. to clean up. ("DllClose()", ...)

        Case $btnVersInfo
          MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION,"Version info", _
            $sVersionNum & @CRLF & @CRLF & "version info example")

      EndSwitch
      ; ------------------------------------------------------------------------
      ; ========================================================================


      ; LinkLabel: detect click events.
      $idx = -1
      For $i = 0 To UBound($aLinkLabels) - 1
        If $aLinkLabels[$i][0] = $iMsg Then
          $idx = $i
          ExitLoop
        EndIf
      Next
      ; Execute commands for LinkLabel clicks.
      If $idx > -1 Then
        ShellExecute($aLinkLabels[$idx][1])
        GUICtrlSetColor($iMsg, $iLinkVisitedColor)
      EndIf

      ; RichEdit: Hide caret and catch tab key.
      If IsFocused($g_hGui, $g_hRichEdit) Then

        ; Hiding the caret of the RichEdit.
        ; If _IsPressed("01", $g_hUser32DLL) Or _IsPressed("02", $g_hUser32DLL) Then ; <= "02": Does NOT work for the right mouse button!
        DllCall($g_hUser32DLL, "int", "HideCaret", "hwnd", $g_hRichEdit) ; <--- this hides caret
        ; EndIf

        ; Wenn man den Tab-Key drückt, wird der gesamte Text markiert und ans Ende
        ; gescrollt. Um das zu verhindern, wird der Tab-Key per "HotKeySet()" abgefangen.
        ; Im Moment ist die Funktion für HotKeySet() eine Dummy-Funktion. 2021-04-08.
        ;
        ; If you press the tab key, the whole text will be selected and scrolled
        ; to the end. To prevent this, the tab key is intercepted by "HotKeySet()".
        ; At the moment the function "TabPress()" is a dummy function. 2021-04-08.
        If $bHotkeyForTabIsSet = False Then
          $bHotkeyForTabIsSet = True
          HotKeySet("{TAB}", "TabPress")
        EndIf
      Else
        If $bHotkeyForTabIsSet = True Then
          $bHotkeyForTabIsSet = False
          HotKeySet("{TAB}")
        EndIf
      EndIf ; If IsFocused($g_hGui, $g_hRichEdit)
    WEnd
;   EndIf ; If _WinAPI_IsClassName($g_hParent, "TfPSPad") ; <= Uncomment if needed.
EndFunc   ;==>Main_Func


; ==============================================================================
; CreateLinkLabel
;
; Routine zum Erstellen der LinkLabels.
;
; Routine to create the LinkLabels.
; ------------------------------------------------------------------------------
Func CreateLinkLabel($_iTop, $_sCaption, $_sUrlOrMailTo)

  ReDim $aLinkLabels[UBound($aLinkLabels) + 1][2]

  $aLinkLabels[UBound($aLinkLabels) - 1][0] = GUICtrlCreateLabel($_sCaption, $iCtrls_Left, $_iTop, $iCtrls_Width, $iCtrls_Height, $SS_CENTER)
  $aLinkLabels[UBound($aLinkLabels) - 1][1] = $_sUrlOrMailTo

  GUICtrlSetFont(-1, -1, -1, $GUI_FONTUNDER) ; underlined
  GUICtrlSetColor(-1, 0x0000ff)              ; blue
  GUICtrlSetCursor(-1, 0)                    ; hand cursor
EndFunc   ;==>CreateLinkLabel


; ==============================================================================
; IsFocused
;
; Routine zum Ermitelln, ob ein bestimmtes Control den Fokus besitzt.
;
; Routine to determine if a certain control has the focus.
;
; Autor: MrCreatoR, 2007-10-01.
; https://www.autoitscript.com/forum/topic/54322-edit-control-tab-key/?do=findComment&comment=411666
; ------------------------------------------------------------------------------
Func IsFocused($hWnd, $nCID)
  Return ControlGetHandle($hWnd, '', $nCID) = ControlGetHandle($hWnd, '', ControlGetFocus($hWnd))
EndFunc   ;==>IsFocused


; ==============================================================================
; OnAutoItExit
;
; Routine die aufgerufen werden soll, wenn das Script beendet wird.
; Sie wurde mit "OnAutoItExitRegister('OnAutoItExit')" registriert.
;
; Routine that should be called when the script is finished.
; It was registered with "OnAutoItExitRegister('OnAutoItExit')".
; ------------------------------------------------------------------------------
Func OnAutoItExit()
  WinSetState($g_hParent, "", @SW_ENABLE) ; Important! Otherwise Parent window is unusable!
EndFunc ; ==> OnAutoItExit


; ==============================================================================
; TabPress
;
; Wenn man den Tab-Key drückt, wird der gesamte Text markiert und ans Ende
; gescrollt. Um das zu verhindern, wird der Tab-Key per "HotKeySet()" abgefangen.
; Im Moment ist die Funktion "TabPress()" eine Dummy-Funktion. 2021-04-08.
;
; If you press the tab key, the whole text will be selected and scrolled to the
; end. To prevent this, the tab key is intercepted by "HotKeySet()". At the
; moment the function "TabPress()" is a dummy function. 2021-04-08.
;
; Autor: MrCreatoR, 2007-10-01.
; https://www.autoitscript.com/forum/topic/54322-edit-control-tab-key/?do=findComment&comment=411666
; ------------------------------------------------------------------------------
Func TabPress()
  ;   Send("^{TAB}")   ;This is the tip from Siao - Thanks!
EndFunc   ;==>TabPress


; ==============================================================================
; WM_COMMAND
;
; Wenn das RichEdit mit dem Scrollbalken gescrollt wird, wird das Caret sichtbar, solange
; der Scrollbalken festgehalten wird. Diese Funktion versteckt das Caret beim Scrollen
; mit dem Scrollbalken. Trotzdem blitzt das Caret beim Scrollen manchmal ein wenig auf.
;
; When the RichEdit is scrolled with the scrollbar, the caret becomes visible as long as
; the scrollbar is held. This function hides the caret when scrolling with the scrollbar.
; Nevertheless, the caret sometimes flashes a little when scrolling.
;
; Tip from pixelsearch, 2021-04-12.
; https://www.autoitscript.com/forum/topic/205604-about-dialog-with-credits-area/?do=findComment&comment=1479582
; ------------------------------------------------------------------------------
Func WM_COMMAND($hWnd, $iMsg, $wParam, $lParam)
  #forceref $hWnd, $iMsg, $wParam

  If $lParam = $g_hRichEdit Then DllCall($g_hUser32DLL, 'int', 'HideCaret', 'hwnd', $g_hRichEdit)

  Return $GUI_RUNDEFMSG
EndFunc   ;==>WM_COMMAND


; ==============================================================================
; WM_NOTIFY
;
; Zum Ermöglichen von klickbare Links im RichEdit.
; Wichtig: Die Links müssen beim Erstellen der RTF-Datei als Links eingerichtet werden!
;
; To enable clickable links in RichEdit.
; Important: The links must be set up as links when creating the RTF file!
;
; From a demo by InunoTaishou, 2016-02-26.
; https://www.autoitscript.com/forum/topic/180900-richedit-hyperlinks-friendly/
; ------------------------------------------------------------------------------
Func WM_NOTIFY($hWnd, $iMsg, $wParam, $lParam)
  #forceref $hWnd, $iMsg, $wParam
  Local $tEnLink = DllStructCreate($tagENLINK, $lParam)
  Local $tNMHDR = DllStructCreate($tagNMHDR, $lParam)
  Local $tMsgFilter = DllStructCreate($tagMSGFILTER, $lParam)

  Switch (DllStructGetData($tNMHDR, "hWndFrom"))
    Case $g_hRichEdit
      Switch (DllStructGetData($tNMHDR, "Code"))
        Case $EN_LINK
          If DllStructGetData($tMsgFilter, "msg") = $WM_LBUTTONUP Then
            ShellExecute(_GUICtrlRichEdit_GetTextInRange($g_hRichEdit, DllStructGetData($tEnLink, "cpMin"), DllStructGetData($tEnLink, "cpMax")))
          EndIf
      EndSwitch
  EndSwitch

  Return $GUI_RUNDEFMSG
EndFunc   ;==>WM_NOTIFY


